/// GCounter with support for state and op changes
export class GCounter {
    _globalCounter;
    // instead of standard incremental id for replicas
    // we map the counter with the node id
    _counts;
    constructor(counts) {
        this._globalCounter = Object.values(counts).reduce((a, b) => a + b, 0);
        this._counts = counts;
    }
    value() {
        return this._globalCounter;
    }
    increment(nodeId, amount) {
        this._globalCounter += amount;
        this._counts[nodeId] += amount;
    }
    counts() {
        return this._counts;
    }
    compare(peerCounter) {
        for (let key in Object.keys(this._counts)) {
            if (this._counts[key] > peerCounter.counts()[key]) {
                return false;
            }
        }
        return true;
    }
    merge(peerCounter) {
        let temp = Object.assign({}, this._counts, peerCounter.counts());
        Object.keys(temp).forEach((key) => {
            this._counts[key] = Math.max(this._counts[key], peerCounter.counts()[key]);
        });
    }
}
